/*! \file checkBeat.c

 \brief Show changes to a beat

checkBeat reads a "beat" file listing packages in a beat.  It then
reads the yum databases for the current Fedora release and the
development repository.  checkBeat compares versions and reports
when versions are different, thus alerting the beat writer to
packages that may need attention for the release notes
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sqlite3.h>

//! Name of the yum database file for the current release
#define DBFILE1 "/home/jjmcd/Projects/checkBeat/F11/primary.sqlite"
//! Name of the yum database file for the development repo
#define DBFILE2 "/home/jjmcd/Projects/checkBeat/rawhide/primary.sqlite"

//! Entry to hold packages, their versions and project URL
struct stEntry
{
  //! Package name
  char szName[64];
  //! Version number of package at initial release
  char szOldVer[64];
  //! Version number of package in development repo
  char szNewVer[64];
  //! URL of the package's project website
  char szURL[256];
  //! Package short description
  char szDesc[256];
} rcEntry[500];

//! File pointer to beat file
FILE *f;
//! Number of entries found in the beat file
int nNumEntries;
//! Buffer for reading the beat file
char szBuffer[1024];
//! Current entry being processed
/*! nCurEntry contains the current entry in rcEntry being
    processsed.  This is a global variable sine the query cannot
    directly pass variables to the callback routines.
*/
int nCurEntry;
//! String used to hold the SQL query
char szQuery[256];
//! Number of entries tested for different versions
int nNumChecked;
//! Number of entries found to have changed
int nNumDiffs;

//! Trim trailing newlines from string
/*! trim() checks for control characters at the end
    of a string and replaces them with null characters.
*/
void trim( char *s )
{
  while ( s[strlen(szBuffer)-1] < ' ' )
    s[strlen(szBuffer)-1] = '\0';
}

//! Callback for getNewVersions
/*! gnvCallback() is called by sqlite in response to a query for
    the version in the development database.  The function simply
    inserts the version into the rcEntry structure.
*/
static int gnvCallback(void *NotUsed, int argc, char **argv, char **azColName)
{
#ifdef DEBUG
  printf("%3d: %s %s\n",nCurEntry,rcEntry[nCurEntry].szName,argv[0]);
#endif
  strcpy(rcEntry[nCurEntry].szNewVer,argv[0]);
}

//! Get newest versions
/*! getNewVersion() opens the development database, then for
    each entry in the rcEntry struct issues a query for the
    package's version.  sqlite returns the result to gnvCallback().
*/
void getNewVersions( void )
{
  sqlite3 *db;
  char *szErrMsg = 0;
  int rc;

  rc = sqlite3_open(DBFILE2,&db);
  if ( rc )
    {         
      fprintf(stderr, "Can't open database: %s\n", sqlite3_errmsg(db));
      sqlite3_close(db);
      exit(1);
    }

  for ( nCurEntry=0; nCurEntry < nNumEntries; nCurEntry++ )
    {
      sprintf(szQuery,"SELECT version FROM packages WHERE name='%s';",
	      rcEntry[nCurEntry].szName);
#ifdef DEBUG
      printf("%3d \"%s\"\n",nCurEntry,szQuery);
#endif
      rc = sqlite3_exec(db, szQuery, gnvCallback, 0, &szErrMsg );
      if ( (rc != SQLITE_OK) && (rc != 4) )
	{
	  fprintf(stderr,"Return from %s is %d.\n",szQuery,rc);
	  fprintf(stderr, "SQL error: %s\n", szErrMsg );
	  sqlite3_free(szErrMsg);
	  exit(1);
	}
      sqlite3_free(szErrMsg);
    }
  sqlite3_close(db);

#ifdef DEBUG
  for ( nCurEntry=0; nCurEntry < nNumEntries; nCurEntry++ )
    {
      printf("%-20s %-20s %-20s\n",rcEntry[nCurEntry].szName,
	     rcEntry[nCurEntry].szOldVer,rcEntry[nCurEntry].szNewVer);
    }
#endif
}

//! Callback for get original versions
/*! govCallback() is called by sqlite in response to a query for
    the version in the original database.  The function simply
    inserts the version and URL into the rcEntry structure.
*/
static int govCallback(void *NotUsed, int argc, char **argv, char **azColName)
{

  printf("%d. %s [%s]\n",argc,azColName[0],argv[0]);
}

//! Get original versions and project's URL
/*! getOriginalVersion() opens the original database, then for
    each entry in the rcEntry struct issues a query for the
    package's version and project URL.  sqlite returns the
    result to govCallback().
*/
void getGroupList( void )
{
  sqlite3 *db;
  char *szErrMsg = 0;
  int rc;
  sqlite3_stmt *ppStmt;
  char *pzTail;
  char szFile[256],szGroup[256];
  char *p;

  rc = sqlite3_open(DBFILE1,&db);
  if ( rc )
    {         
      fprintf(stderr, "Can't open database: %s\n", sqlite3_errmsg(db));
      sqlite3_close(db);
      exit(1);
    }

      sprintf(szQuery,"SELECT DISTINCT rpm_group FROM packages ORDER BY rpm_group;");

      rc = sqlite3_prepare(db,szQuery,sizeof(szQuery),&ppStmt,&pzTail);

      //rc = sqlite3_exec(db, szQuery, govCallback, 0, &szErrMsg );
      if ( (rc != SQLITE_OK) && (rc != 4) )
	{
	  fprintf(stderr,"Return from %s is %d.\n",szQuery,rc);
	  fprintf(stderr, "SQL error: %s\n", szErrMsg );
	  sqlite3_free(szErrMsg);
	  exit(1);
	}
      rc = 0;

      while ( sqlite3_step( ppStmt ) != SQLITE_DONE )
	{
	  strcpy(szGroup,sqlite3_column_text(ppStmt,0));
	  strcpy(szFile,szGroup);
	  p = szFile;
	  while ( *p )
	    {
	      if ( *p == ' ' )
		*p = '_';
	      if ( *p == '/' )
		*p = '-';
	      p++;
	    }
	  printf("echo '.output %s.beat' >tmp.ini\n",szFile);
	  printf("sqlite3 -init tmp.ini ~/Projects/checkBeat/F11/primary.sqlite \"SELECT name FROM packages WHERE rpm_group='%s' ORDER BY NAME;\"\n",
		 szGroup);
	  printf("echo ' ' >>temp.mw\n");
	  printf("echo '==%s==' >>temp.mw\n",szGroup);
	  printf("pushd ..\n");
	  printf("./checkBeat12 'groups/%s.beat' >>groups/temp.mw\n",szFile);
	  printf("popd\n");
	}
      /*      rc = sqlite3_step( ppStmt );
      if ( (rc != SQLITE_OK) && (rc != 4) && (rc != SQLITE_DONE) )
	{
	  fprintf(stderr,"Return from step is %d.\n",szQuery,rc);
	  fprintf(stderr, "SQL error: %s\n", szErrMsg );

	  sqlite3_free(szErrMsg);
	  exit(1);
	  }*/
      sqlite3_free(szErrMsg);
  sqlite3_close(db);

}


//! Compare versions of packages in beat and display
/* compareVersions() goes through the rcEntry structure
   and compares the new and original versions.  If they are
   different it displays the package name, old and new
   versions, and the project's URL.
*/
void compareVersions( void )
{
#ifdef DEBUG
  int i;
#endif

  //  printf("Package,Cat1,Cat2,Cat3,Summary,\"Old Ver\",\"New Ver\"\r\n");
  printf("<center>'''Changed Packages'''</center>\n\n\n");
  printf("{|style=\"width:100%%\"\n");
  printf("|-\n! Package || <center>Fedora 11</center> || <center>Fedora 12</center> || Upstream URL\n");
  nNumChecked = nNumDiffs = 0;
  for ( nCurEntry=0; nCurEntry < nNumEntries; nCurEntry++ )
    {
      nNumChecked++;
      if ( strcmp(rcEntry[nCurEntry].szOldVer,rcEntry[nCurEntry].szNewVer) )
	{
	  //printf("\"%s\",,,,\"%s\",\"%s\",\"%s\",%s\r\n",rcEntry[nCurEntry].szName,
	  //	 rcEntry[nCurEntry].szDesc,
	  //	 rcEntry[nCurEntry].szOldVer,rcEntry[nCurEntry].szNewVer,
	  //	 rcEntry[nCurEntry].szURL);
	  printf("|-\n| {{Package|%s}} ",
		 rcEntry[nCurEntry].szName);
	  if ( strlen(rcEntry[nCurEntry].szOldVer)>0 )
	    printf("|| <center>%s</center> ",rcEntry[nCurEntry].szOldVer);
	  else
	    printf("|| <center>new</center> ");
	  if ( strlen(rcEntry[nCurEntry].szNewVer)>0 )
	    printf("|| <center>%s</center> ",rcEntry[nCurEntry].szNewVer);
	  else
	    printf("|| <center>dropped</center> ");
	  if ( strlen(rcEntry[nCurEntry].szURL)>0 )
	    printf("|| %s \n",rcEntry[nCurEntry].szURL);
	  else
	    printf("|| &nbsp; \n");

	  nNumDiffs++;
#ifdef DEBUG
	  printf("\t");
	  for (i=0; i<strlen(rcEntry[nCurEntry].szOldVer); i++)
	    printf("%3d ",rcEntry[nCurEntry].szOldVer[i]);
	  printf("\n");
	  printf("\t");
	  for (i=0; i<strlen(rcEntry[nCurEntry].szNewVer); i++)
	    printf("%3d ",rcEntry[nCurEntry].szNewVer[i]);
	  printf("\n");
#endif
	}
      //else
	//printf("%s-%s is unchanged.\n",rcEntry[nCurEntry].szName,
	//       rcEntry[nCurEntry].szOldVer);
    }
  printf("|}\r\n");
  printf("\n\n%d packages, %d changed, %d unchanged.\n\n\n",nNumChecked,nNumDiffs,nNumChecked-nNumDiffs);
}

//! Compare versions in two primary.sqlite files for a beat
/* checkBeat main() tests the number of arguments to ensure the
   user has provided a beat file, then calls buildList(),
   getOriginalVersions(), getNewVersions() and compareVersions().
*/
int main( int argc, char *argv[] )
{

  getGroupList();
  //  getNewVersions();
  //  compareVersions();

  return 0;
}
