/*! \file doc-publican-rpm.c

  \brief Make a multi-language RPM from a Publican build tree.

  This file contains \c main() for the application.

  The user provides a configuration file which lists a Publican
  directory containing the document to be packaged.  The user must
  have built some html formatted documents.

  The application looks through the tmp directory tree for htmls,
  and on finding them, uses the corresponding xml to locate the
  appropriate text for the menu in the local language.

  A tar, spec, rpm and srcrpm are produced in the 
  <tt>~/rpmbuild tree</tt>.

  Usage:

  - The rpmbuild environment must be installed and set up.
  - A Publican document must be available having at least a
    few languages in html including en-US.
  - An empty scratch directory must be manually created.
  - That scratch directory must contain the
    <tt>doc-publican-rpm.config</tt> file.
  - That scratch directory must be the default when 
    \p doc-publican-rpm is executed.

  The <tt>doc-publican-rpm.config</tt> file contains all the information
  \p doc-publican-rpm needs to create the rpm file.  The program 
  takes no arguments.

  The <tt>doc-publican-rpm.config</tt> file contains five lines:
  - The author's name - this is used in the changelog.
  - The author's email - this is also used in the changelog.
  - The Publican directory.  All the html files will be gathered
    from <tt>tmp/<lang>/html</tt> under this directory.
  - The version number for the rpm.
  - The release number for the rpm.

  If an existing package is being rebuilt, the user may provide the
  previous spec file, renamed as <tt>.old</tt>.  If the file is found,
  doc-publican-rpm will use the description and changelog from that
  old spec file.

  doc-publican-rpm will create the following files:
  - <tt>~/rpmbuild/RPMS/noarch/<name>-<version>-<release>.fcx.noarch.rpm</tt>
  - <tt>~/rpmbuild/SRPMS/<name>-<version>-<release>.fcx.src.rpm</tt>
  - <tt>~/rpmbuild/SOURCES/<name>-<version>.tar.gz</tt>
  - <tt>./<name>.spec</tt>

  The <tt><name></tt> segment of all those file is taken from the portion
  of the provided Publican directory following the final slash.

\dot
digraph Context
{
size="4.0,4.0"
rankdir="LR";
fontname="Helvetica-Bold";
label="doc-publican-rpm context";
node [ shape="record" style="filled" fillcolor="lightyellow" 
       fontname="Helvetica" ];
"doc-publican-rpm" [ shape="circle" style="filled" fillcolor="lightcyan" ];
"Publican Files" -> "doc-publican-rpm";
"doc-publican-rpm.config" -> "doc-publican-rpm";
"old .spec\n(optional)" -> "doc-publican-rpm";
"doc-publican-rpm" -> "tar";
"doc-publican-rpm" -> ".spec"
"doc-publican-rpm" -> "SRPM";
"doc-publican-rpm" -> "RPM";
}
\enddot
*/

/*   jjmcd - 2009-12-06 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define configfile "doc-publican-rpm.config"

//! Version number of the produced rpm
char szVersion[20];
//! Release number of the produced rpm
char szRelease[16];
//! Publican directory
char szPubDir[512];
//char szBuildTree[512];
//! Buffer for file operations
char szBuffer[1024];

//! Base name for the rpm and document
char szBasename[64];
//! Directory where source tarfile will be staged
char szWorkdir[256];
//! Commands sent to system
char szCommand[256];
//! Author's name
char szAuthor[128];
//! Author's email address
char szEmail[128];
//! Number of languages to process
int nEntry;

//! Structure containing per-language information
struct
{
  //! ISO code for the language
  char szLang[16];
  //! Document title in local language
  char szTitle[128];
  //! Document subtitle in local language
  char szSubtitle[512];
} rcConvInfo[50];

void getBaseName( void );
void getLanguages( char * );
void readConfigFile( char * );
void readArticleInfo( char *, char *, char *, char * );
void copyFiles( void );
void makeDesktopFile( char * );
void makeIndexFile( char * );
void makeTarFile( void );
void makeSpecFile( void );

/*! main() - Main control for the application

  main() mostly calls the other functions that comprise the application.
  main() is linear, all decisions are made at lower levels.  main() does
  have responsibility for creating the top of the directory structure
  which will ultimately produce the source tar.

Pseudocode:
\code
Read Configuration File
Get base RPM name from Publican directory
Look through tmp directory to identify languages
Copy documents to a working tree
For each Language
  Read the corresponding Article_Info.xml
Create the desktop file
Create the index.html file
Creat the source tar from the working tree
Create the .spec file
Run rpmbuild
\endcode
*/
int main()
{
  int i;

  readConfigFile(configfile);
  getBaseName();
  getLanguages( szPubDir );
  copyFiles();


  for ( i=0; i<nEntry; i++ )
    readArticleInfo( szPubDir,
		     rcConvInfo[i].szLang,
		     rcConvInfo[i].szTitle, 
		     rcConvInfo[i].szSubtitle );

  makeDesktopFile( szWorkdir );
  makeIndexFile( szWorkdir );
  makeTarFile();
  makeSpecFile();

  return 0;
}
