/*! \file makeDesktopFile.c

    \brief Write out the .desktop file for menu

    Using the titles and subtitles read by getArticleInfo()
    generates the <tt>.desktop</tt> file containing entries for all
    the included languages.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

extern int nEntry;
extern struct
{
  char szLang[16];
  char szTitle[128];
  char szSubtitle[512];
} rcConvInfo[50];
extern char szBasename[64];
extern char szVersion[20];
void cleanString( char *, char * );

/*! cvtLang - Convert language code

    Publican produces documents using the RFC4646 language-locale code
    similar to ll-LL where ll is the ISO 639-1 language code and LL is
    the ISO 3166-1 location code (with possible mdifiers).  However,
    Gnome requires a different standard, ll_LL.  This function
    converts the Publican code to the Gnome code.

*/
void cvtLang( char *in, char *out )
{
  strcpy(out, in );
  if ( out[2] == '-' )
    out[2]  = '_';
}

/*! makeDesktopFile() - Create the .desktop file

    Generates the <tt>.desktop</tt> file for the document.  Loops
    through the rcConvInfo structure to get the text for the various
    languages.  The file is created in the desktop directory of the
    staging directory for the tar file.  Does the same thing with a
    few minor differences for KDE.

    Argument:
    \arg szDir - Directory where tar file is being staged.
*/
void makeDesktopFile( char *szDir )
{
  char szFilename[512];
  FILE *f;
  int i;
  char szCLang[16];
  char szTextString[1024];

  // Make the Gnome file
  strcpy(szFilename,szDir);
  strcat(szFilename,"/desktop/");
  strcat(szFilename,szBasename);
  strcat(szFilename,".desktop");

  f = fopen(szFilename,"w+");
  if ( f == NULL )
    {
      perror("creating desktop file");
      exit(1);
    }
  fprintf(f,"[Desktop Entry]\n");
  // desktop-file-validate doesn't like this
  // fprintf(f,"Encoding=UTF-8\n");
  cleanString(rcConvInfo[0].szTitle,szTextString);
  fprintf(f,"Name=%s\n",szTextString);
  for ( i=1; i<nEntry; i++ )
    {
      cvtLang(rcConvInfo[i].szLang ,szCLang);
      cleanString(rcConvInfo[i].szTitle,szTextString);
      fprintf(f,"Name[%s]=%s\n",
	      szCLang,szTextString);
    }
  cleanString(rcConvInfo[0].szTitle,szTextString);
  fprintf(f,"GenericName=%s\n",szTextString);
  cleanString(rcConvInfo[0].szSubtitle,szTextString);
  fprintf(f,"Comment=%s\n",szTextString);
  for ( i=1; i<nEntry; i++ )
    {
      cvtLang(rcConvInfo[i].szLang ,szCLang);
      cleanString(rcConvInfo[i].szSubtitle,szTextString);
      fprintf(f,"Comment[%s]=%s\n",
	      szCLang,szTextString);
    }
  fprintf(f,"Exec=htmlview /usr/share/doc/HTML/%s/index.html\n",
	  szBasename);
  // BZ587704 Use new icon instead of Publican icon
  fprintf(f,"Icon=fedora-documentation\n");
  fprintf(f,"Terminal=false\n");
  fprintf(f,"Type=Application\n");
  // BZ583284 - Added System; at front
  // Taking out System; causes duplicate menu
  fprintf(f,"Categories=Documentation;X-Red-Hat-Base;\n");
  // BZ583284 - unnecessary so rpmlint bitches
  //fprintf(f,"Version=%s\n",szVersion);
  fprintf(f,"StartupNotify=true\n");
  // Without this is (sometimes) shows up in KDE
  fprintf(f,"OnlyShowIn=GNOME;XFCE;LXDE;\n");
  // BZ482947
  //fprintf(f,"X-Desktop-File-Install-Version=0.15\n");
  fclose(f);

  strcpy(szFilename,szDir);
  strcat(szFilename,"/desktop/");
  strcat(szFilename,szBasename);
  strcat(szFilename,".KDE.desktop");

  // Make the KDE file
  f = fopen(szFilename,"w+");
  if ( f == NULL )
    {
      perror("creating desktop file");
      exit(1);
    }
  fprintf(f,"[Desktop Entry]\n");
  //fprintf(f,"Encoding=UTF-8\n");
  cleanString(rcConvInfo[0].szSubtitle,szTextString);
  fprintf(f,"Name=%s\n",szTextString);
  for ( i=1; i<nEntry; i++ )
    {
      cleanString(rcConvInfo[i].szSubtitle,szTextString);
      cvtLang(rcConvInfo[i].szLang ,szCLang);
      fprintf(f,"Name[%s]=%s\n",
	      szCLang,szTextString);
    }
  // Eliminating Comment causes KDE to lack the second line
  // of the description.  Let's see if we can get away with
  // Name and Comment without GenericName, and lets' make
  // the Name be the Title instead of Subtitle
  // Also BZ482947
  cleanString(rcConvInfo[0].szTitle,szTextString);
  fprintf(f,"GenericName=%s\n",szTextString);
  for ( i=1; i<nEntry; i++ )
    {
      cvtLang(rcConvInfo[i].szLang ,szCLang);
      cleanString(rcConvInfo[i].szTitle,szTextString);
      fprintf(f,"GenericName[%s]=%s\n",
  	      szCLang,szTextString);
    }
  // desktop-file-validate doesn't like having this
  // duplicate stuff here, not sure how much we will
  // suffer if we drop this
   /* cleanString(rcConvInfo[0].szSubtitle,szTextString);  */
   /* fprintf(f,"Comment=%s\n",szTextString);  */
   /* for ( i=1; i<nEntry; i++ )  */
   /*   {  */
   /*     cleanString(rcConvInfo[i].szSubtitle,szTextString);  */
   /*     cvtLang(rcConvInfo[i].szLang ,szCLang);  */
   /*     fprintf(f,"Comment[%s]=%s\n",  */
   /* 	      szCLang,szTextString);  */
   /*   }  */
  fprintf(f,"Exec=htmlview /usr/share/doc/HTML/%s/index.html\n",
	  szBasename);
  // BZ587704 Use new icon instead of Publican icon
  fprintf(f,"Icon=fedora-documentation\n");
  fprintf(f,"Terminal=false\n");
  fprintf(f,"Type=Application\n");
  fprintf(f,"OnlyShowIn=KDE;\n");
  fprintf(f,"Categories=Documentation;System;\n");
  // BZ583284 - unnecessary so rpmlint bitches
  //fprintf(f,"Version=%s\n",szVersion);
  fprintf(f,"X-KDE-StartupNotify=true\n");
  // BZ482947
  //fprintf(f,"X-Desktop-File-Install-Version=0.15\n");
  fclose(f);

}

