/*! \file makeIndexFile.c

  \brief Create the main index.html

  The target directory tree contains a directory for each language.
  The base file for each language is <tt>index.html</tt> in that
  language's directory.  An <tt>index.html</tt> file above that
  directory has the sole purpose of selecting the appropriate
  <tt>index.html</tt> for the user's preferred language.  This
  function creates that <tt>index.html</tt>, which needs a list of
  available languages.

    Pseudocode for index.html:
\code
Get browser's lang-loc
Search for match in list of lang-loc's available
if not found
    search for match against language only in lang-loc list
    if not found
        set lang-loc to en-US
window.location = ./<lang-loc>/index.html
\endcode

 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

extern int nEntry;
extern struct
{
  char szLang[16];
  char szTitle[128];
  char szSubtitle[512];
} rcConvInfo[50];
extern char szBasename[64];
extern char szVersion[20];
char szLangs[50][16];

/*! makeIndexFile() - Create the index.html

    Generates an <tt>index.html</tt> which uses the browser language
    to determine which html to display.  The document's menu item
    always points to this file.

    Arguments:
    \arg szDir - pointer to the directory to write the file

*/
void makeIndexFile( char *szDir )
{
  char szFilename[512];
  char szHold[16];
  int nSwap;
  FILE *f;
  int i;

  strcpy(szFilename,szDir);
  strcat(szFilename,"/index.html");

  f = fopen(szFilename,"w+");
  if ( f == NULL )
    {
      perror("creating index file");
      exit(1);
    }

  fprintf(f,"<html>\n");
  fprintf(f,"<head>\n");
  fprintf(f,"<script language=\"JavaScript1.2\">\n");
  fprintf(f,"<!-- Begin\n");
  fprintf(f,"if (navigator.appName == 'Netscape')\n");
  fprintf(f,"  var l1 = navigator.language;\n");
  fprintf(f,"else\n");
  fprintf(f,"  var l1 = navigator.browserLanguage;\n");
  fprintf(f,"-->\n");
  fprintf(f,"</script>\n");
  fprintf(f,"\n");
  fprintf(f,"<title>%s</title>\n",
	  rcConvInfo[0].szTitle);
  fprintf(f,"<body>\n");
  fprintf(f,"<script type=\"text/javascript\">\n");
  fprintf(f,"  var lang = l1;\n");
  fprintf(f,"  var match = 0;\n");
  fprintf(f,"  var locales = [");
  for ( i=1; i<nEntry; i++ )
    {
      if ( i != 1 )
	fprintf(f,", ");
      fprintf(f,"\"%s\"",rcConvInfo[i].szLang);
    }
  fprintf(f,"];\n");
  fprintf(f,"\n");
  fprintf(f,"  // Try for full match of lang_LOC\n");
  fprintf(f,"  for(locale in locales)\n");
  fprintf(f,"    {\n");
  fprintf(f,"      if(lang == locales[locale])\n");
  fprintf(f,"        {\n");
  fprintf(f,"          match = 1;\n");
  fprintf(f,"          break;\n");
  fprintf(f,"        }\n");
  fprintf(f,"    }\n");
  fprintf(f,"\n");
  fprintf(f,"  // Else try for match of lang\n");
  fprintf(f,"  if(match == 0)\n");
  fprintf(f,"    {\n");
  fprintf(f,"      for(locale in locales)\n");
  fprintf(f,"        {\n");
  fprintf(f,"          var loc_lang = locales[locale].substring(0,2);\n");
  fprintf(f,"          var language = lang.substring(0,2);\n");
  fprintf(f,"\n");
  fprintf(f,"          if(loc_lang == language)\n");
  fprintf(f,"            {\n");
  fprintf(f,"              lang = locales[locale];\n");
  fprintf(f,"              match = 1;\n");
  fprintf(f,"              break;\n");
  fprintf(f,"            }\n");
  fprintf(f,"        }\n");
  fprintf(f,"    }\n");
  fprintf(f,"\n");
  fprintf(f,"  // Else default to en-US\n");
  fprintf(f,"  if(match == 0)\n");
  fprintf(f,"    {\n");
  fprintf(f,"      lang = 'en-US';\n");
  fprintf(f,"    }\n");
  fprintf(f,"\n");
  fprintf(f,"  window.location = lang + \"/index.html\";\n");
  fprintf(f,"</script>\n");
  // For Javascript disabled
  fprintf(f,"</noscript>\n<p>\n<ul>\n<ul>\n");
  for ( i=0; i<nEntry; i++ )
    strcpy(szLangs[i],rcConvInfo[i].szLang);
  nSwap = 1;
  while ( nSwap )
    {
      nSwap = 0;
      for ( i=1; i<nEntry; i++ )
	{
	  if ( strcmp(szLangs[i],szLangs[i-1]) < 0 )
	    {
	      strcpy(szHold,szLangs[i-1]);
	      strcpy(szLangs[i-1],szLangs[i]);
	      strcpy(szLangs[i],szHold);
	      nSwap++;
	    }
        }
    }
  for ( i=0; i<nEntry; i++ )
      fprintf(f,"<li><a href=\"./%s/index.html\">%s</a></li>\n",
	      szLangs[i],szLangs[i]);
  fprintf(f,"</ul>\n</ul>\n</p>\n</noscript>\n");
  fprintf(f,"</body>\n");
  fprintf(f,"</html>\n");
  fclose(f);

}
