/*! \file makeSpecFile.c

  \brief Write out a spec file for the document

  The generated <tt>.spec</tt> file is aimed at Fedora.  Publican
  produced specfiles include some conditionalization which essentially
  selects a different launching application form RHEL.  However, the
  <tt>.desktop</tt> file cannot be conditionalized in that way, so the
  specfile also removes that conditionalization so as not to mislead
  the reader.


  Specfile install peudocode:
\code
Make /usr/share/doc/HTML/<basename> directory
For all languages in the rpm
    Call doAdir for /usr/share/doc/HTML/<basename>/<lang> directory:
      Create directory
      Call self for subdirectories
      Install any files
Install /usr/share/doc/HTML/<basename>/index.html
Install /usr/share/applications/<basename>.desktop
\endcode
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <dirent.h>

void cleanString( char *, char *);

extern char szVersion[20];
extern char szRelease[16];
extern char szBasename[64];
extern char szAuthor[128];
extern char szEmail[128];
extern char szPubDir[512];
extern struct
{
  char szLang[16];
  char szTitle[128];
  char szSubtitle[512];
} rcConvInfo[50];

/*! doAdir() - Process a directory

doAdir() processes a single directory.  A mkdir command is emitted for
the directory.  Then the files in the directory are examined and
doAdir() emits install commands for any regular files found.  If a sub
directory is located, doAdir() calls itself for that subdirectory.

The result is that the mkdir commands for the top level directories
will be first, but the installs for that directory will be last.

\param f - File pointer to open file for output
\param szBase - Symbolic "install from" location
\param szTarg - Symbolic "install to" location
\param szWhere - Publican directory being processed.

doAdir() pseudocode:
\code
Open directory (opendir())
Allocate memory for directory entry
Emit command to make subdirectory
for each file in the directory
  if the file isn't . or ..
    if the file is a directory
      call doAdir() to process directory
    else
      make note of extension
for each extension found
  install files of that extension
close directory
free memory
\endcode

*/
void doAdir( FILE *f, char *szBase, char *szTarg, char *szWhere )
{
  DIR *dir;
  struct dirent *de, *entry;
  char szNewWhere[512];
  char szNewTarg[512];
  char szNewBase[512];
  int nEntry;
  char szExts[32][8];
  char szLine[256];
  int nNumExts;
  int i;
  char *p;
  int nFound;

  dir = opendir(szWhere);
  nEntry = 0;
  nNumExts = 0;

  entry = malloc(sizeof(struct dirent));

  // Explicitly create this directory
  fprintf(f,"  mkdir -p %s\n", szTarg );
  fflush(f);

  do
    {
      // Regular readdir() doesn't seem to play nice here.  Apparently
      // reentrant counts as multithread from readdir()'s perspective
      readdir_r ( dir, entry, &de );
      if ( de != NULL )
	{
	  if ( strcmp(de->d_name,".") && strcmp(de->d_name,"..") )
	    {
	      if ( de->d_type == DT_DIR )
		{
		  strcpy( szNewBase, szBase );
		  strcat( szNewBase, "/" );
		  strcat( szNewBase, de->d_name );
		  strcpy( szNewWhere, szWhere );
		  strcat( szNewWhere, "/" );
		  strcat( szNewWhere, de->d_name );
		  strcpy( szNewTarg, szTarg );
		  strcat( szNewTarg, "/" );
		  strcat( szNewTarg, de->d_name );
		  doAdir( f, szNewBase, szNewTarg, szNewWhere );
		}
	      else if ( de->d_type == DT_REG )
		{
		  // Found a file to install, remember its extension
		  nEntry++;
		  p = strrchr(de->d_name,'.');
		  if ( p )
		    {
		      p++;
		      nFound = 0;
		      for ( i=0; i<nNumExts; i++ )
			// but only unique extensions
			if ( !strcmp(p,szExts[i]) )
			     nFound++;
		      if ( !nFound )
			{
			  strcpy(szExts[nNumExts],p);
			  nNumExts++;
			}
		    }
		}
	    }
	}
    } while ( de != NULL );
  // If there were files (and not just directories), install them
  if ( nEntry )
    {
      // fprintf(f,"  # Files in %s\n",szTarg);
      for ( i=0; i<nNumExts; i++ )
	{
	  sprintf(szLine,"  install -m 644 %s/*.%s %s/\n",
		  szBase, szExts[i], szTarg);
	  if ( strlen(szLine) > 70 )
	    sprintf(szLine,"  install -m 644 %s/*.%s \\\n    %s/\n",
		    szBase, szExts[i], szTarg);
	  fprintf(f,szLine);
	}
    }
  // Make note in html.lang
  fprintf(f,"  echo  \"${NOTETARG}/\" >> html.lang\n");

  // Now clean up the debris
  closedir( dir );
  free(entry);
}

/*! getChangelog() - retrieve the changelog from earlier specfile

    getChangelog()  attempts to open a file named (docname).spec.old in
    the working directory. If no file is found, it returns false.  If
    the file is opened successfully, the file is read until 
    <tt>%%changelog</tt> is located.  The remainder of the file is then 
    copied to the new spec file being created.

    the function accepts the name of the new spec file as a parameter,
    adds <tt>.old</tt> to it, and attempts to open that file.  If the file
    does not contain a changelog, no changelog will appear in the
    new spec file.

    \param szSpecName - the name of the newly created specfile
    \param f - pointer to the open file
    \returns - 0 if no file opened, else 1
 */
int getChangelog( char *szSpecName, FILE *f )
{
  char szOldName[128];
  FILE *g;
  char szBuffer[512];

  strcpy(szOldName,szSpecName);
  strcat(szOldName,".old");
  g = fopen( szOldName, "r" );
  if ( g == NULL )
    return 0;
  szBuffer[0]='X';
  while(strncmp("%changelo",szBuffer,8) )
    fgets(szBuffer,sizeof(szBuffer),g);
  szBuffer[0]='X';
  while(!feof(g))
    {
      fgets(szBuffer,sizeof(szBuffer),g);
      if ( !feof(g) )
	fputs(szBuffer,f);
    }
  fclose(g);
  return 1;
}


/*! getOldDesc() - retrieve the description from earlier specfile

    getOldDesc()  attempts to open a file named (docname).spec.old in
    the working directory. If no file is found, it returns false.  If
    the file is opened successfully, the file is read until 
    <tt>%%description</tt> is located.  The file is then copied to the
    new file until a line beginning with a "%" is found.

    The function accepts the name of the new spec file as a parameter,
    adds <tt>.old</tt> to it, and attempts to open that file.  If the file
    does not contain a description, no description will appear in the
    new spec file.

    \param szSpecName - the name of the newly created specfile
    \param f - pointer to the open file
    \returns - 0 if no file opened, else 1
 */
int getOldDesc( char *szSpecName, FILE *f )
{
  char szOldName[128];
  FILE *g;
  char szBuffer[512];

  strcpy(szOldName,szSpecName);
  strcat(szOldName,".old");
  g = fopen( szOldName, "r" );
  if ( g == NULL )
    return 0;
  szBuffer[0]='X';
  while(strncmp("%descrip",szBuffer,8) )
    fgets(szBuffer,sizeof(szBuffer),g);
  szBuffer[0]='X';
  while(szBuffer[0]!='%')
    {
      fgets(szBuffer,sizeof(szBuffer),g);
      if ( szBuffer[0]!='%' )
	fputs(szBuffer,f);
    }
  fclose(g);
  return 1;
}


/*! makeSpecFile() - Write out a .spec file for the document.

makeSpecFile() writes out a fairly generic <tt>.spec</tt> file.  The
only real changes needed between documents are the
name/version/release and the information for the changelog.

If an old spec file exists, makeSpecFile() uses the description
and changelog from the old file, rather than providing a generic
description and changelog.

makeSpecFile() then goes ahead and executes <tt>rpmbuild</tt> to
generate the rpm and source rpm.  
*/
void makeSpecFile( void )
{
  time_t tCurrentTime;
  char szCurrentTime[256];
  char szSpecName[128];
  FILE *f;
  char szCommand[256];
  char szDir[512];
  char szCleanDesc[512];

  tCurrentTime = time(NULL);
  strcpy(szCurrentTime,ctime(&tCurrentTime));
  szCurrentTime[strlen(szCurrentTime)-1] = '\0';
  strcpy(&szCurrentTime[11],&szCurrentTime[20]);

  strcpy(szSpecName,szBasename);
  strcat(szSpecName,".spec");
  f = fopen(szSpecName,"w+");
  if ( f == NULL )
    {
      perror("creating specfile");
      exit(1);
    }
  fprintf(f,"# Documentation Specfile\n\n");

  fprintf(f,"Name:      %s\n",szBasename);
  fprintf(f,"Version:   %s\n",szVersion);
  fprintf(f,"Release:   %s%%{?dist}\n",szRelease);
  fprintf(f,"Summary:   %s\n",rcConvInfo[0].szTitle);
  fprintf(f,"URL:       http://fedoraproject.org\n");

  fprintf(f,"Group:     System Environment/Base\n");
  fprintf(f,"License:   CC-BY-SA\n");
  fprintf(f,"Source0:   %s-%%{version}.tar.gz\n",szBasename);
  fprintf(f,"BuildRoot: %%{_tmppath}/%%{name}-%%{version}-%%{release}-root-%%(%%{__id_u} -n)\n");
  fprintf(f,"BuildArch: noarch\n");
  //fprintf(f,"Requires:\t/usr/bin/htmlview\n");
  // BZ583284 - unnecessary so rpmlint bitches
  //fprintf(f,"Provides:\t%s = %%{version}-%%{release}\n\n",szBasename);
  fprintf(f,"Requires:\tcoreutils\n");
  fprintf(f,"Requires(post):\t/bin/touch\n");

  fprintf(f,"\n%%description\n");
  if ( !getOldDesc( szSpecName, f ) )
    {
      cleanString(rcConvInfo[0].szSubtitle,szCleanDesc);
      fprintf(f,"%s\n",szCleanDesc);
      fprintf(f,"\nA Publican document from the Fedora Documentation Project.\n");
      fprintf(f,"\n");
    }

  fprintf(f,"%%prep\n");
  fprintf(f,"%%setup -q\n");
  fprintf(f,"\n");

  fprintf(f,"%%build\n");
  fprintf(f,"\n");

  fprintf(f,"%%install\n");
  fprintf(f,"rm -rf $RPM_BUILD_ROOT\n");
  fprintf(f,"mkdir -p $RPM_BUILD_ROOT%%{_defaultdocdir}/HTML\n");
  fprintf(f,"mkdir -p $RPM_BUILD_ROOT%%{_defaultdocdir}/HTML/%%{name}\n");

  fprintf(f,"#\n");
  fprintf(f,"# Loop through the languages\n");
  fprintf(f,"#\n");
  fprintf(f,"cat /dev/null > html.lang\n");
  fprintf(f,"for LANGDIR in document/* ; do\n");
  fprintf(f,"  #\n");
  fprintf(f,"  # First, the html in /usr/share/doc/HTML, then descend\n");
  fprintf(f,"  # through any subdirectories, installing everything found.\n");
  fprintf(f,"  #\n");
  fprintf(f,"  # Language to process\n");
  fprintf(f,"  LANG=${LANGDIR#document/}\n");
  fprintf(f,"  # Target for the html\n");
  fprintf(f,"  NOTETARG=$RPM_BUILD_ROOT%%{_defaultdocdir}/HTML/%%{name}/${LANG}\n");
  fprintf(f,"  # Place where html files are\n");
  fprintf(f,"  SRCBASE=${LANGDIR}\n");
  strcpy(szDir,szPubDir);
  strcat(szDir,"/tmp/en-US/html");
  doAdir( f, "${SRCBASE}", "${NOTETARG}", szDir );
  fprintf(f,"done\n");

  fprintf(f,"#\n");
  fprintf(f,"# index.html\n");
  fprintf(f,"#\n");
  fprintf(f,"install -m 644 index.html $RPM_BUILD_ROOT%%{_defaultdocdir}/HTML/%%{name}\n");
  fprintf(f,"#\n");
  fprintf(f,"# Now the desktop files\n");
  fprintf(f,"#\n");
  fprintf(f,"mkdir -p $RPM_BUILD_ROOT%%{_datadir}/applications\n");
  fprintf(f,"mkdir -p $RPM_BUILD_ROOT%%{_datadir}/applications/kde4\n");
  fprintf(f,"SRCBASE=desktop\n");
  fprintf(f,"install -m 644 ${SRCBASE}/%s.desktop \\\n    $RPM_BUILD_ROOT%%{_datadir}/applications\n",szBasename);
  fprintf(f,"install -m 644 ${SRCBASE}/%s.KDE.desktop \\\n    $RPM_BUILD_ROOT%%{_datadir}/applications/kde4/%s.desktop\n",szBasename,szBasename);
  //fprintf(f,"SRCBASE=tmp/${LANG}/xml\n");
  fprintf(f,"\n");
  fprintf(f,"#%%find_lang %%{name} --with-gnome --all-name\n");
  fprintf(f,"#for F in $RPM_BUILD_ROOT%%{_defaultdocdir}/HTML/index-*.html ; do\n");
  fprintf(f,"#  L=`echo ${F} | %%{__sed} 's/.*\\/index-\\(.*\\)\\.html$/\\1/'`\n");
  fprintf(f,"#  echo \"%%%%lang(${L}) ${F#$RPM_BUILD_ROOT}\" >> html.lang\n");
  fprintf(f,"#done\n");
  fprintf(f,"cat html.lang >> %%{name}.lang\n");
  fprintf(f,"\n");
  fprintf(f,"#\n# Icons\n#\n");
  fprintf(f,"ICONDIR=\"$RPM_BUILD_ROOT%%{_datadir}/icons/hicolor/\"\n");
  //fprintf(f,"mkdir -p ${ICONDIR}/scalable/apps\n");
  //fprintf(f,"install -m 644 document/en-US/images/icon.svg \\\n");
  //fprintf(f,"  ${ICONDIR}/scalable/apps/fedora-documentation.svg\n");
  fprintf(f,"for SZ in icons/* ; do\n");
  fprintf(f,"  SZA=${SZ#icons/}\n");
  fprintf(f,"  mkdir -p $RPM_BUILD_ROOT%%{_datadir}/icons/hicolor/${SZA}/apps \n");
  fprintf(f,"  install -m 644 ${SZ}/apps/fedora-documentation.png \\\n");
  fprintf(f,"    ${ICONDIR}/${SZA}/apps/fedora-documentation.png \n");
  fprintf(f,"done\n\n");

  fprintf(f,"%%clean\n");
  fprintf(f,"rm -rf $RPM_BUILD_ROOT\n");
  fprintf(f,"\n");
  fprintf(f,"%%post\n");
  fprintf(f,"if [ -x /usr/bin/scrollkeeper-update ]; then scrollkeeper-update -q; fi\n");
  fprintf(f,"    touch --no-create /usr/share/icons/hicolor\n");
  fprintf(f,"if [ -x /usr/bin/gtk-update-icon-cache ]; then\n");
  fprintf(f,"    gtk-update-icon-cache -q /usr/share/icons/hicolor &> /dev/null || :\n");
  fprintf(f,"fi\n");
  fprintf(f,"if [ -x /usr/bin/update-desktop-database ]; then update-desktop-database &> /dev/null; fi\n");
  fprintf(f,"\n");
  fprintf(f,"%%postun\n");
  fprintf(f,"if [ -x /usr/bin/scrollkeeper-update ]; then scrollkeeper-update -q; fi\n");
  fprintf(f,"    touch --no-create /usr/share/icons/hicolor\n");
  fprintf(f,"if [ -x /usr/bin/gtk-update-icon-cache ]; then\n");
  fprintf(f,"    gtk-update-icon-cache -q /usr/share/icons/hicolor &> /dev/null || :\n");
  fprintf(f,"fi\n");
  fprintf(f,"if [ -x /usr/bin/scrollkeeper-update ]; then scrollkeeper-update -q; fi\n");
  fprintf(f,"if [ -x /usr/bin/update-desktop-database ]; then update-desktop-database &> /dev/null; fi\n");
  fprintf(f,"\n");
  fprintf(f,"%%files\n");
  fprintf(f,"%%defattr(-,root,root,-)\n");
  fprintf(f,"%%{_defaultdocdir}/HTML\n");
  fprintf(f,"%%{_datadir}/applications/%s.desktop\n",szBasename);
  fprintf(f,"%%{_datadir}/applications/kde4/%s.desktop\n",szBasename);
  fprintf(f,"%%{_datadir}/icons/hicolor/16x16/apps/fedora-documentation.png\n");
  fprintf(f,"%%{_datadir}/icons/hicolor/22x22/apps/fedora-documentation.png\n");
  fprintf(f,"%%{_datadir}/icons/hicolor/24x24/apps/fedora-documentation.png\n");
  fprintf(f,"%%{_datadir}/icons/hicolor/32x32/apps/fedora-documentation.png\n");
  fprintf(f,"%%{_datadir}/icons/hicolor/48x48/apps/fedora-documentation.png\n");
  //  fprintf(f,"%%{_datadir}/icons/hicolor/scalable/apps/fedora-documentation.svg\n");

  fprintf(f,"\n");
  fprintf(f,"%%changelog\n");
  if ( !getChangelog( szSpecName, f ) )
    {
      fprintf(f,"* %s %s <%s> - %s-%s\n",szCurrentTime,szAuthor,szEmail,szVersion,szRelease);
      fprintf(f,"- Auto generated specfile\n");
      fprintf(f,"\n");
    }

  fclose(f);

  strcpy(szCommand,"cp ");
  strcat(szCommand,szSpecName);
  strcat(szCommand," ~/rpmbuild/SPECS");
  system(szCommand);

  strcpy(szCommand,"rpmbuild --quiet -ba ");
  strcat(szCommand,szSpecName);
  //strcat(szCommand," >rpmbuild.log");
  printf("Running rpmbuild.\n");
  system(szCommand);

}
